using System;
namespace Entity
{
	/// <summary>
	/// Representa una coleccion de una Entidad para un Entity.Entidad
	/// </summary>
	/// 
	[Serializable]
	public class ListaEntidades : Entity.ICollectionBase 
	{
		public Entidad this[int index]
		{
			get { return ((Entidad)(List[index])); } 
			set { List[index] = value; }
		}

		/// <summary>
		/// Obtiene un objeto Entity.Entidad que corresponda con el parametro proporcionado
		/// </summary>
		/// <param name="entidad">identifica el numero del estado</param>
		/// <returns>una entidad de Entity.Entidad</returns>
		public Entidad FindByPrimaryKey(int entidad)
		{
			System.Collections.IEnumerator elementos=List.GetEnumerator();
			Entidad elemento=null;

			while(elementos.MoveNext())
			{
				elemento=(Entidad)elementos.Current;
				if(elemento.Estado==entidad)
				{
					return elemento;
				}
			}
			return null;
		}

	}

	/// <summary>
	/// Representa una entidad de una Entidad para su trabajo en memoria
	/// </summary>
	[Serializable]
	public class Entidad : IEntityBase
	{
		private int entidad;
		private Pais pais;
		private string descripcion;
		
		public Entidad()
		{			
		}

		/// <summary>
		/// Determina si las instancias de Entity.Entidad se consideran iguales
		/// </summary>
		/// 
		public override bool Equals(object obj)
		{
			return (this.entidad == ((Entidad)obj).entidad );
		}
 
		/// <summary>
		/// Sirve como una funcion hash para un tipo concreto, apropiado para su utilizacion en algoritmos de hash y en estructuras de datos como las tablas hash
		/// </summary>
		/// <returns>numero de codigo hash</returns>
		public override int GetHashCode()
		{
			return base.GetHashCode ();
		}
		/// <summary>
		/// Obtiene la descripcion de un Entity.Entidad
		/// </summary>
		/// <returns></returns>
		public override string ToString()
		{
			return this.descripcion;
		}

		/// <summary>
		/// Obtiene o establece un Guid para identificar la entidad para un Entity.Entidad
		/// </summary>
		public int Estado
		{
			get{return this.entidad;}
			set
			{
				if(value!=this.entidad)
				{
					this.Set();
					this.entidad=value;
				}
			}
		}
		/// <summary>
		/// Obtiene o establece un Entity.Pais para un Entity.Entidad
		/// </summary>
		public Pais Pais
		{
			get{return this.pais;}
			set
			{
				if(this.pais == null || (this.pais != null && !this.pais.Equals(value))) 
				{
					this.Set();
					this.pais=value;
				}
			}
		}
		/// <summary>
		/// Obtiene o establece una descripcion para un Entity.Entidad
		/// </summary>
		public string Descripcion
		{
			get{return this.descripcion;}
			set
			{
				if(value!=this.descripcion)
				{
					this.Set();
					this.descripcion=value;
				}
			}
		}
	}
}
