using System;

namespace Entity
{
	/// <summary>
	/// Representa una coleccion de un Pais para un Entity.Pais
	/// </summary>
	/// 
	[Serializable]
	public class ListaPaises : Entity.ICollectionBase 
	{
		public Pais this[int index]
		{
			get { return ((Pais)(List[index])); } 
			set { List[index] = value; }
		}

		/// <summary>
		/// Obtiene el objeto Entity.Pais de la coleccion que coincida con los parametros proporcionados
		/// </summary>
		/// <param name="paisID">Guid que identifica el pais</param>
		/// <returns>una entidad de Entity.Pais</returns>
		public Pais FindByPrimaryKey(int paisId)
		{
			System.Collections.IEnumerator elementos=List.GetEnumerator();
			Pais elemento=null;

			while(elementos.MoveNext())
			{
				elemento=(Pais)elementos.Current;
				if(elemento.PaisId==paisId)
				{
					return elemento;
				}
			}
			return null;
		}

	}

	/// <summary>
	/// Representa una entidad de un Pais para su trabajo en memoria
	/// </summary>
	[Serializable]
	public class Pais : IEntityBase
	{
		private int paisId;
		private string descripcion;

		public Pais()
		{
			paisId=0;
		}

		/// <summary>
		/// Obtiene o Establece un Guid que identifica el pais para un Entity.Pais
		/// </summary>
		public int PaisId
		{
			get{return this.paisId;}
			set
			{
				if(value!=this.paisId)
				{
					this.Set();
					this.paisId=value;
				}
			}
		}
		/// <summary>
		/// Obtiene o Establece una descripcion para un Entity.Pais
		/// </summary>
		public string Descripcion
		{
			get{return this.descripcion;}
			set
			{
				if(value!=this.descripcion) 
				{
					this.Set();
					this.descripcion=value;
				}
			}
		}

		/// <summary>
		/// Determina si las instancias de Entity.Pais se consideran iguales
		/// </summary>
		/// 
		public override bool Equals(object obj)
		{
			return (this.paisId == ((Pais)obj).paisId );
		}
 
		/// <summary>
		/// Sirve como una funcion hash para un tipo concreto, apropiado para su utilizacion en algoritmos de hash y en estructuras de datos como las tablas hash
		/// </summary>
		/// <returns>numero de codigo hash</returns>
		public override int GetHashCode()
		{
			return base.GetHashCode ();
		}
		/// <summary>
		/// Obtiene la descripcion de un Entity.Pais
		/// </summary>
		/// <returns>el valor del atributo descripcion</returns>
		public override string ToString()
		{
			return this.descripcion;
		}
	}
}
