using System;
using System.Xml;
namespace Entity
{
	/// <summary>
	/// Controla los datos de los parametros del archivo parametros.xml, como la conexion a la base de datos, directorios de DBFs, etc.
	/// </summary>
	public class Parametros
	{
		private string connection;
		private volatile static Parametros instance=null; //para la utilizacion del patrn Singleton
		
		/// <summary>
		/// Obtiene o establece un string de conexion para la base de datos para un Entity.Parametros
		/// </summary>
		public string Conecction
		{
			get { return this.connection;}
			set { this.connection =value;}
		}
		
		/// <summary>
		/// Obtiene un objeto Entity.Parametros
		/// </summary>
		/// <returns>la instancia unica de Entity.Parametros utilizando el patron Singleton de gamma</returns>
		public static Parametros GetInstance()
		{
			if(instance==null)
			{
				lock(typeof(Parametros))
				{
					if(instance==null)
					{
						instance=new Parametros();
					}
				}
			}
			return instance;
		}

		/// <summary>
		/// Recarga los valores del archivo parametros.xml sobre los atributos de la clase
		/// </summary>
		public void Recargar()
		{
			XmlDocument parametros= new XmlDocument();
			parametros.Load("Parametros.xml");
			this.connection =parametros.SelectSingleNode("/NewDataSet/Parametros/parametro[@id='connection']").InnerText;			
			Entity.LogController.Instance().AgregarMensaje("Cargue los parametros " + DateTime.Now.ToString(new System.Globalization.DateTimeFormatInfo()) ,false);
		}

		/// <summary>
		/// Constructor que llena los atributos
		/// </summary>
		protected Parametros()
		{
			Recargar();	
		}
	}
}
